#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>

#include "dvrcardApi.h"
#include "tmserverpu.h"

#define PREVIEW

#ifdef PREVIEW
#include <X11/Xlib.h>
static SDL_Surface *gSDL_Surface = NULL;
static SDL_Overlay *pOverlayScreen = NULL;
static SDL_Rect dstRect;
int win_width = 801;
int win_height = 601;
#endif

unsigned short format1[] = {16,19,'O','f','f','i','c','e','\0'};
unsigned short format2[] = {8,3,_OSD_YEAR4,':',_OSD_MONTH2,':',_OSD_HOUR24,':',_OSD_MINUTE,':',_OSD_SECOND,'\0'};

int  gcheckpass(char * usename, int namelen, char * password, int passlen);
int gcheckip(int iChannel, char * sIP);
void gmessage(char * buf, int iLen);
void callback(int channelNum,char * buf,int frametype,int length);	

int audio_port = 0;
static int done = 0;
void poked(int sig)
{
	done = 1;
}

int main()
{
	int channelCount;
	int i,retVal, chan;
	int channelHandle[16];
	SERVER_VIDEOINFO videoinfo;
	RECT winRect;
	STREAM_READ_CALLBACK pStreamRead;
	pStreamRead = (STREAM_READ_CALLBACK)callback;	

	int tei=0;
	char data[4];

	signal(SIGHUP, poked);
	signal(SIGINT, poked);
	signal(SIGTERM, poked);

#ifdef PREVIEW
	XInitThreads();

	//putenv("SDL_VIDEO_YUV_DIRECT=1");
	putenv("SDL_VIDEO_YUV_HWACCEL=0");
	if(SDL_Init(SDL_INIT_VIDEO) < 0){
		fprintf(stderr,"SDL initial video failed !\n");
		exit(0);
	}
	else{
		unsigned int videoFlag = 0;
		const SDL_VideoInfo *sdl_videoInfo;
		
		sdl_videoInfo = SDL_GetVideoInfo();
		if(sdl_videoInfo->hw_available)
			videoFlag |= SDL_HWSURFACE;
		
		videoFlag |= SDL_DOUBLEBUF;
		gSDL_Surface = SDL_SetVideoMode(win_width, win_height, 0, videoFlag);
		if(gSDL_Surface == NULL){
			fprintf(stderr,"SDL_SetVideoMode() failed\n");
			exit(1);
		}
	}
#endif

	retVal = InitDSPs();
	if(retVal<0){
		fprintf(stderr,"Do InitDSPs Error(%08x)\n",GetLastErrorNum());
		return 0;
	}
	fprintf(stderr,"Success!----InitDSPs()\n");
	
	channelCount = GetTotalChannels();
	fprintf(stderr,"channelcount = %d\n",channelCount);

	videoinfo.m_channum = channelCount;					
	MP4_ServerSetNetPort(5050,6050);

	MP4_ServerStart(&videoinfo);
	MP4_ServerSetBufNum(1,60);
	
	printf("videoinfo at 0x%x, m_channum at 0x%x\n", &videoinfo, &videoinfo.m_channum);
	
	MP4_ServerCheckPassword(gcheckpass);
	MP4_ServerCheckIP(gcheckip);
	MP4_ServerSetMessage(gmessage);	

	//open channel	
	for(i=0;i<channelCount;i++){
		channelHandle[i] = ChannelOpen( i ,callback);
	}

	sleep(1);
	for(chan=0;chan<channelCount;chan++){
#ifdef PREVIEW
		//start video preview
		if(channelCount == 1){
			dstRect.w = win_width;
			dstRect.h = win_height;
			dstRect.x = 0;
			dstRect.y = 0;
		}
		else if(channelCount == 2){
			dstRect.w = win_width;
			dstRect.h = win_height/2;
			dstRect.x = 0;
			dstRect.y = dstRect.h * (chan%2);
		}
		else if(((channelCount - 1)/4) == 0){// <=4 channels
			dstRect.w = win_width/2;
			dstRect.h = win_height/2;
			dstRect.x = dstRect.w * (chan%2);
			dstRect.y = dstRect.h * (chan/2);
		}
		else if(channelCount <= 16){
			// >4 channels, <=16 channels
			dstRect.w = win_width/4;
			dstRect.h = win_height/4;
			dstRect.x = dstRect.w * (chan%4);
			dstRect.y = dstRect.h * (chan/4);
		}
		fprintf(stderr,"dstRect.x %d, dstRect.y %d, w = %d, h = %d\n", dstRect.x, dstRect.y,dstRect.w,dstRect.h);
		
		if(StartVideoPreview(channelHandle[chan], gSDL_Surface, dstRect) < 0){
			fprintf(stderr,"strat video preview failed, error 0x%lx\n", GetLastErrorNum());
		}
#endif
		usleep(40);
		SetOsdDisplayMode(channelHandle[chan],0xff,1,0x0000, format1, format2);
		SetOsd(channelHandle[chan],1);
	}//for(chan=0;cahn<channelCount;chan++)

	//SetEncodePictureFormat(channelHandle[1],ENC_QCIF_FORMAT);
	sleep(1);
	SetAudioPreview(channelHandle[audio_port], 1);
	//start capture
	for(i=0;i<channelCount;i++){
		StartVideoCapture(channelHandle[i]);
	}
	//start	motion detect
	for(i=0;i<channelCount;i++){
		StartMotionDetection(channelHandle[i]);
	}
	while(!done){
		sleep(1);
	}
	fprintf(stderr,"beging stop!\n");
	MP4_ServerStop();
	//stop preview
	fprintf(stderr,"stop video!\n");
	for(i=0;i<channelCount;i++){
		StopVideoPreview(channelHandle[i]);
	}
	fprintf(stderr,"stop audio!\n");
	SetAudioPreview(channelHandle[audio_port],0);
	fprintf(stderr,"stop motion detection!\n");
	//stop motion detect
	for(i = 0; i < channelCount; i++)
	{
		StopMotionDetection(channelHandle[i]);
	}
	fprintf(stderr,"stop video capture!\n");
	//stop capture
	for(i=0;i<channelCount;i++){
		StopVideoCapture(channelHandle[i]);
	}
	fprintf(stderr,"close channel!\n");
	//close channel
	for(i=0;i<channelCount;i++){
		ChannelClose(channelHandle[i]);
	}
	fprintf(stderr,"deinit dsp!\n");
	retVal = DeInitDSPs();
	if(retVal<0){
		fprintf(stderr,"Do DeInitDSPs Error(%08x)\n",GetLastErrorNum());					      
		return 0;
	}

#ifdef PREVIEW
		SDL_Quit();
#endif
	return 0;
}


void callback(int channelNum,char * buf,int frametype,int length)
{
	int retval;

	switch(frametype)
	{
		case PktSysHeader:
		case PktIFrames:
		case PktPFrames:
		case PktBBPFrames:
		case PktAudioFrames:
		{
			if(channelNum == 0)
				;//fprintf(stderr,"frametype = 0x%x\n",frametype);
			retval = MP4_ServerWriteData(channelNum,(char *)buf,length,frametype,0);
#if 0
			if(retval==0){
				if(0x0001 == frametype)
				      fprintf(stderr,"*%d", channelNum);
			}
			else{
				if(channelNum == 0)
					fprintf(stderr,".");
			}
#endif
			break;
		}
		case PktMotionDetection:
			break;
		default:
			break;
	}
}

int  gcheckpass(char * usename, int namelen, char * password, int passlen)
{
	fprintf(stderr,"i am in checkpass\n");
	fprintf(stderr,"usename=%s\n,pass=%s\n",usename,password);
	fprintf(stderr,"lenname=%d,passlen=%d\n",namelen,passlen);
	return 0;
}

int gcheckip(int iChannel, char * sIP)
{
	fprintf(stderr,"i am in checkip\n");
	fprintf(stderr,"ichannel=%d,ip=%s\n",iChannel,sIP);
	
	return 0;
}
void gmessage(char * buf, int iLen)
{
	fprintf(stderr,"i am in messagecallback\n");
	fprintf(stderr,"len=%d,message=%s\n",iLen,buf);
}

