/*
 * h264play.c - H.264 player 
 */

#include "h264play.h"
#include <unistd.h>
#include <fcntl.h>
#include <SDL.h>

//#define FILE_POS

#define TIMER_CONST 100


GtkWidget *h264_window;
GdkFont *fixed_font;
static int EndFlag = 0;

int timer = 0;
int totaltime = 0;
int totalframes = 0;

int portno = 0;
int using_sdl = 1;	//1 - will play audio and video
int play_audio = 1;
int capture_picture = 0;

int OpenFlag = 0;	//add at 2002.9.13

int videofd;

int FileRefFlag = 0;

static void h264_set_sensitive(char *name, int sensitive);
static void h264_set_buttons_sensitive(int sensitive);

static gint h264_key_press(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	//printf("key press\n");
	return 0;
}

void ToggleFullScreen()
{
	SDL_Surface *screen;
	screen = SDL_GetVideoSurface();
	if(SDL_WM_ToggleFullScreen(screen)){
		printf("Toggled full screen - now %s\n", screen->flags & SDL_FULLSCREEN ? "fullscreen":"windowed");
	}
}

static void FileEndCallBack(int port)
{
	//EndFlag = 1;	
}

static void DecCallBack(int port, char *pBuf, int size, FRAME_INFO *FrameInfo,long reserved1, long reserved2 )
{
	printf("size = %5d, width = %3ld, height = %3ld, stamp = %4ld, fram_rate = %2ld\n", size, FrameInfo->width, FrameInfo->height, FrameInfo->stamp, FrameInfo->frame_rate);
	if((FrameInfo->frame_rate) && (videofd))
		write(videofd, pBuf, size);
}

static void DisplayCallBack(int port, char *pBuf, long size, long width, long height, long stamp, long type,long reserved)
{
	char filename [100];
		
	if(!capture_picture)
		return;
	sprintf(filename,"%ld.bmp", stamp);
	PlayM4_ConvertToBmpFile(pBuf, size, width, height, type, filename);
	capture_picture = 0;
}

static void FileRefCallBack(int port)
{
	//printf("finish setup file index\n");
	FileRefFlag = 1;
}

void play_audio_callback(GtkWidget *widget, gpointer data)
{
	play_audio = (GTK_TOGGLE_BUTTON(widget)->active);
	if(play_audio){
		if(PlayM4_PlaySound(portno) < 0){
			printf("Play sound failed, ErrorCode =  0x%lx.\n", (unsigned long)PlayM4_GetLastErrorCode());
		}
	}else{
		PlayM4_StopSound(portno);
	}
}


static void h264_set_time(int value)
{
	GtkWidget *w_currenttime;
	char buf[256];
	int current_hour, current_min, current_sec;
	current_hour= value/3600;
	current_min = (value%3600)/60;
	current_sec = value%60;
	sprintf(buf, "%02d:%02d:%02d", current_hour, current_min, current_sec);
	
	w_currenttime = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(h264_window), "currenttime"));
	gtk_text_backward_delete(GTK_TEXT(w_currenttime), strlen(buf));
	gtk_text_insert(GTK_TEXT(w_currenttime), fixed_font/*NULL*/, NULL, NULL, buf, -1);
}

static  int h264_timer(gpointer data)
{
	GtkWidget *scale;
	GtkAdjustment *seek;
	GtkWidget *w_frameplayed;
	static int playedtime = 0;
	static int playedframes = 0;
	char buf[100];

	SDL_Event event;
	PLAYRECT  rect;

	if(FileRefFlag)	
		h264_set_sensitive("stepback", 1);
	else
		h264_set_sensitive("stepback", 0);

	while(SDL_PollEvent(&event) && using_sdl){
		switch(event.type){
			case SDL_VIDEORESIZE:
				//printf("SDL resize event: 0x%x, w = %d, h = %d\n", event.type, event.resize.w, event.resize.h);
				PlayM4_SetDisplay(event.resize.w, event.resize.h);
				rect.x = 0;
				rect.y = 0;
				rect.uWidth = event.resize.w;
				rect.uHeight = event.resize.h;
				PlayM4_SetPlayRect(portno, rect);
				break;
			case SDL_KEYDOWN:
				if(event.key.keysym.sym == SDLK_f)
					ToggleFullScreen();
				//printf("SDL: key press\n");
				break;
			default:
				break;
		}
	}

	playedtime  = PlayM4_GetPlayedTime(portno);
	scale = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(h264_window), "scale"));
	seek = gtk_range_get_adjustment(GTK_RANGE(scale));
	h264_set_time(playedtime);
#ifdef FILE_POS
	retval = (int)(PlayM4_GetPlayPos(portno) * 100);
	if(retval < 0)
		seek->value = 0;
	else seek->value = retval;
#else
	seek->value = (playedtime * 100)/totaltime;
#endif
	gtk_range_set_adjustment(GTK_RANGE(scale), seek);
	gtk_range_slider_update(GTK_RANGE(scale));
	//gtk_range_clear_background(GTK_RANGE(scale));
	//gtk_range_draw_background(GTK_RANGE(scale));
	playedframes = PlayM4_GetCurrentFrameNum(portno);
	//printf("playedframes = %d\n", playedframes);
	w_frameplayed = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(h264_window), "frameplayed"));
	gtk_text_backward_delete(GTK_TEXT(w_frameplayed), gtk_text_get_length(GTK_TEXT(w_frameplayed)));
	//printf("length of w_frameplayed = %d\n", gtk_text_get_length(GTK_TEXT(w_frameplayed)));
	sprintf(buf, "%8d", playedframes);
	gtk_text_insert(GTK_TEXT(w_frameplayed), fixed_font/*NULL*/, NULL, NULL, buf, -1);
	if(EndFlag){
		if(PlayM4_Stop(portno) < 0){
			return 1;
		}
		h264_set_buttons_sensitive(0);
		h264_set_sensitive("play", 1);
		EndFlag = 0;
	}
	return 1; /* keep call this function */
}

void my_exit(GtkWidget *w, GdkEvent *event, gpointer data)
{
	
	gtk_main_quit();
}

static void h264_dialog_cleanup(GtkWidget *dialog)
{

	gtk_grab_remove(dialog);
	gtk_widget_destroy(dialog);
}

static void h264_dialog_cancel(GtkWidget *widget, GtkWidget *dialog)
{
	assert(dialog);
	h264_dialog_cleanup(dialog);
}

/* buttons operation functions */
static void h264_set_sensitive(char *name, int sensitive)
{
	GtkWidget *button;
	button = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(h264_window), name));
	gtk_widget_set_sensitive(button, sensitive);
}

static void h264_set_buttons_sensitive(int sensitive)
{
	h264_set_sensitive("play", sensitive);
	h264_set_sensitive("pause", sensitive);
	h264_set_sensitive("stop", sensitive);
	
	h264_set_sensitive("goto_start", sensitive);
	h264_set_sensitive("back", sensitive);
	h264_set_sensitive("forward", sensitive);
	h264_set_sensitive("goto_end", sensitive);

	if((sensitive && FileRefFlag) || (!sensitive))	
		h264_set_sensitive("stepback", sensitive);
}

static void h264_play(GtkWidget *widget, gpointer data)
{
	PLAYRECT rect;
	void (*pFileEndCallBack)();
	void (*pDecCallBack)();
	void (*pDisplayCallBack)();

	if(!using_sdl){
		pDecCallBack = DecCallBack;
		PlayM4_SetDecCallBack(portno, pDecCallBack);
	}
	rect.x = 0;
	rect.y = 0;
	rect.uWidth = 352;
	rect.uHeight = 288;
	
	PlayM4_GetPictureSize(portno, &rect.uWidth, &rect.uHeight);

	if(rect.uWidth == 704)//2CIF
		rect.uHeight *= 2;
	
	PlayM4_SetDisplay(rect.uWidth, rect.uHeight);
		
	pFileEndCallBack = FileEndCallBack;
	PlayM4_SetFileEndCallBack(portno, pFileEndCallBack);

	pDisplayCallBack = DisplayCallBack;
	PlayM4_SetDisplayCallBack(portno, pDisplayCallBack);
	
	if(play_audio){
		if(PlayM4_PlaySound(portno) < 0){
			printf("Play sound failed, ErrorCode =  0x%lx.\n", (unsigned long)PlayM4_GetLastErrorCode());
		}
	}

	if(PlayM4_Play(portno, rect) < 0){
		printf("Play failed, ErrorCode =  0x%lx.\n", (unsigned long)PlayM4_GetLastErrorCode());
		return;
	}
	
	h264_set_buttons_sensitive(1);
	h264_set_sensitive("play", 0);

	EndFlag = 0;
}

static void h264_pause(GtkWidget *widget, gpointer data)
{
	static int enable = 0;

	enable  ^= 1;
	if(PlayM4_Pause(portno, enable) < 0)
		return;
	
}

static void h264_stop(GtkWidget *widget, gpointer data)
{
	if(play_audio)	
		PlayM4_StopSound(portno);
	if(PlayM4_Stop(portno) < 0)
	{	
		return;
	}
	h264_set_buttons_sensitive(0);
	h264_set_sensitive("play", 1);
}

static void h264_goto_start(GtkWidget *widget, gpointer data)
{
	if(FileRefFlag)
		PlayM4_SetCurrentFrameNum(portno, 0);
	PlayM4_SetPlayPos(portno, 0);
}

static void h264_back(GtkWidget *widget, gpointer data)
{
	PlayM4_Slow(portno);
	h264_set_sensitive("play", 1);
}

static void h264_forward(GtkWidget *widget, gpointer data)
{
	PlayM4_Fast(portno);
	h264_set_sensitive("play", 1);
}

static void h264_goto_end(GtkWidget *widget, gpointer data)
{
	if(FileRefFlag){
		PlayM4_SetCurrentFrameNum(portno, totalframes - 1);
	}
	PlayM4_SetPlayPos(portno, 1);
}

static void h264_stepback(GtkWidget *widget, gpointer data)
{
	PlayM4_OneByOneBack(portno);
	h264_set_sensitive("play", 1);
}

static void h264_step(GtkWidget *widget, gpointer data)
{
	PlayM4_OneByOne(portno);
	h264_set_sensitive("play", 1);
}

static void h264_picture(GtkWidget *widget, gpointer data)
{
	capture_picture= 1;
}
/* end of buttons operation functions */


static void h264_open_ok(GtkWidget *widget, GtkWidget *fs)
{
	GtkWidget *w_totaltime;
	gchar *filename;
	int total_hour, total_min, total_sec;
	GtkWidget *w_totalframes;
	char buf[256];
	unsigned int pic_quality;
	void (*pFileRefCallBack)();
	
	INITINFO init_info;

	if(OpenFlag)
		PlayM4_CloseFile(portno);
	
	init_info.uWidth = 352;
	init_info.uHeight = 288;
	if(PlayM4_Init(init_info, using_sdl) < 0)
	{
		printf("Initial failed\n");
		gtk_exit(1);
	}

	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	
	/*create file index */
	FileRefFlag = 0;
	pFileRefCallBack = FileRefCallBack;
	PlayM4_SetFileRefCallBack(portno, pFileRefCallBack);	
	
	/* open file */
	if(PlayM4_OpenFile(portno, filename) < 0){
		printf("	Open file failed, ErrorCode = 0x%lx.\n", (unsigned long)PlayM4_GetLastErrorCode());
		PlayM4_CloseFile(portno);
		gtk_exit(1);
	}
	
	PlayM4_ThrowBFrameNum(portno, 0);
	PlayM4_SetPicQuality(portno, 0);
	PlayM4_GetPicQuality(portno, &pic_quality);
	//printf("picture quality = %d\n", pic_quality);

	OpenFlag = 1;
	
	h264_dialog_cleanup(GTK_WIDGET(fs)); /* moved here at 2002.7.17 */

	/* display total time */	
	totaltime = PlayM4_GetFileTime(0);
	if(!totaltime){
		printf("Total time of file is 0!\n");
		return;
	}
	total_hour= totaltime/3600;
	total_min = (totaltime%3600)/60;
	total_sec = totaltime%60;
	sprintf(buf, "%02d:%02d:%02d", total_hour, total_min, total_sec);
	
	w_totaltime = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(h264_window), "totaltime"));
	gtk_text_backward_delete(GTK_TEXT(w_totaltime), strlen(buf));
	gtk_text_insert(GTK_TEXT(w_totaltime), fixed_font/*NULL*/, NULL, NULL, buf, -1);

	/* dispaly total frames */
	totalframes = PlayM4_GetFileFrames(portno);
	w_totalframes = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(h264_window), "totalframes"));
	gtk_text_backward_delete(GTK_TEXT(w_totalframes), gtk_text_get_length(GTK_TEXT(w_totalframes)));
	sprintf(buf, "%8d", totalframes);
	gtk_text_insert(GTK_TEXT(w_totalframes), fixed_font/*NULL*/, NULL, NULL, buf, -1);
		
	h264_set_buttons_sensitive(1);

	h264_play(NULL, NULL);

	timer = gtk_timeout_add(TIMER_CONST, h264_timer, NULL); /* start timer */
}

static void h264_close(GtkWidget *widget, gpointer data)
{
	PlayM4_CloseFile(portno);
	OpenFlag = 0;
	FileRefFlag = 0;
	h264_set_buttons_sensitive(0);
	if( timer){
		gtk_timeout_remove(timer);
		timer = 0;
	}
}

static void h264_open(GtkWidget *widget, gpointer raw)
{
	GtkWidget *file_sel = NULL;

	h264_close(NULL,NULL);
	
	file_sel = gtk_file_selection_new("Select a file to play");
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->ok_button), 
			"clicked", GTK_SIGNAL_FUNC(h264_open_ok), file_sel);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel)->cancel_button), 
			"clicked", GTK_SIGNAL_FUNC(h264_dialog_cancel), file_sel);
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_sel));

	gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_sel), "test000.mp4");

	gtk_widget_show(file_sel);
	
}


static void h264_quit(GtkWidget *widget, gpointer data)
{
	h264_close(NULL, NULL);
	

	PlayM4_DeInit();

	gdk_font_unref(fixed_font);	
	gtk_main_quit();
}


static void h264_about(GtkWidget *widget, gpointer data)
{
	printf("h264_about.\n");
}


static GtkItemFactoryEntry menu_items[] = {
	{"/File(_F)", 		NULL, 		NULL, 		0, 	"<Branch>"},
	{"/File(_F)/Open", 	"<control>O", 	h264_open, 	0, 	NULL},
	{"/File(_F)/Close", 	"<control>C", 	h264_close, 	0, 	NULL},
	{"/File(_F)/Quit", 	"<control>Q", 	h264_quit, 	0, 	NULL},
	{"/Help(_H)", 		NULL, 		NULL, 		0, 	"<Branch>"},
	{"/Help(_H)/About(_A)", NULL,		h264_about, 	0, 	NULL},
	

};

GtkWidget *get_main_menu(GtkWidget *widget)
{
	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;
	int menu_items_size = sizeof(menu_items)/sizeof(menu_items[0]);

	accel_group = gtk_accel_group_new();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_group);
	gtk_item_factory_create_items(item_factory,
			menu_items_size, menu_items, NULL);
	gtk_window_add_accel_group(GTK_WINDOW(widget), accel_group);
	
	return gtk_item_factory_get_widget(item_factory, "<main>");
}

GtkWidget *xpm_box(GtkWidget *parent, gchar **xpm_data)
{
	GtkWidget *pixmapwid;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GtkStyle *style;

	assert(parent);
	style = gtk_widget_get_style(parent);
	pixmap = gdk_pixmap_create_from_xpm_d(parent->window,
			&mask,
			&style->bg[GTK_STATE_NORMAL],
			xpm_data);
	pixmapwid = gtk_pixmap_new(pixmap, mask);
	gtk_widget_show(pixmapwid);
	return pixmapwid;
}


/* callback functions */
void ch_volume(GtkAdjustment *adjust, gpointer data)
{
	int volume = adjust->value;
	PlayM4_SetVolume(portno, volume);
}

void ch_seek(GtkAdjustment *adjust, gpointer data)
{
	
	PlayM4_SetPlayPos(portno, (float)adjust->value/100.0);
}

GtkWidget * h264_create_window()
{
	//GtkWidget *h264_window;
	GtkWidget *main_box;
	GtkWidget *menubar;
	GtkWidget *table;
	GtkWidget *play, *button_box_play;
	GtkWidget *pause, *button_box_pause;
	GtkWidget *stop, *button_box_stop;
	GtkWidget *space;
	GtkWidget *goto_start, *button_box_goto_start;
	GtkWidget *button_box_back;
	GtkWidget *forward, *button_box_forward;
	GtkWidget *goto_end, *button_box_goto_end;
	GtkWidget *back;
	GtkWidget *step;
	GtkWidget *picture;
	GtkObject *seek;
        GtkWidget *scale;
        GtkWidget *label;
        GtkObject *volume;
        GtkWidget *volume_scale;
	GtkWidget *w_currenttime;
	GtkWidget *w_totaltime;
	GtkWidget *w_frameplayed;
	GtkWidget *w_totalframes;
	GtkWidget *audio;

	char buf[100];
	int tmp;

	fixed_font = gdk_font_load("-misc-fixed-medium-r-*-*-*-120-*-*-*-*-*-*");	

	h264_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(h264_window), "H.264 PLAYER");
	gtk_signal_connect(GTK_OBJECT(h264_window), "destroy",
			GTK_SIGNAL_FUNC(h264_quit), NULL);
	gtk_signal_connect(GTK_OBJECT(h264_window), "key_press_event",
			GTK_SIGNAL_FUNC(h264_key_press), NULL);
	gtk_widget_set_uposition(GTK_WIDGET(h264_window), 0, 0);
	gtk_widget_show(h264_window);

	main_box = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(h264_window), main_box);
	gtk_widget_show(main_box);
	
	/* create mebu bar */
	menubar = get_main_menu(h264_window);
	gtk_box_pack_start(GTK_BOX(main_box), menubar, FALSE, FALSE, 0);
	gtk_widget_show(menubar);
		
	/* create a 5x16 table */
	table = gtk_table_new(5, 16, FALSE/*TRUE*/);
	gtk_box_pack_start(GTK_BOX(main_box), table, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(table), 10);
	gtk_widget_show(table);

	/* create play operation buttons */
	/* play button */	
	play = gtk_button_new();
	button_box_play = xpm_box(h264_window, (gchar **)xpm_play);
	gtk_container_add(GTK_CONTAINER(play), button_box_play);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "play", play,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(play), "clicked",
			GTK_SIGNAL_FUNC(h264_play), NULL);
	gtk_table_attach(GTK_TABLE(table), play, 0, 1, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(play);
	
	
	/* pause button */	
	pause = gtk_button_new();
	button_box_pause = xpm_box(h264_window, (gchar **)xpm_pause);
	gtk_container_add(GTK_CONTAINER(pause), button_box_pause);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "pause", pause,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(pause), "clicked",
			GTK_SIGNAL_FUNC(h264_pause), NULL);
	gtk_table_attach(GTK_TABLE(table), pause, 1, 2, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(pause);
	
	
	/* stop button */	
	stop = gtk_button_new();
	button_box_stop = xpm_box(h264_window, (gchar **)xpm_stop);
	gtk_container_add(GTK_CONTAINER(stop), button_box_stop);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "stop", stop,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(stop), "clicked",
			GTK_SIGNAL_FUNC(h264_stop), NULL);
	gtk_table_attach(GTK_TABLE(table), stop, 2, 3, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(stop);
	
	/* separate */	
	space = xpm_box(h264_window, (gchar **)xpm_space);
	gtk_table_attach(GTK_TABLE(table), space, 3, 4, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(space);
	
	/* goto_start button */	
	goto_start = gtk_button_new();
	button_box_goto_start = xpm_box(h264_window, (gchar **)xpm_goto_start);
	gtk_container_add(GTK_CONTAINER(goto_start), button_box_goto_start);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "goto_start", goto_start,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(goto_start), "clicked",
			GTK_SIGNAL_FUNC(h264_goto_start), NULL);
	gtk_table_attach(GTK_TABLE(table), goto_start, 4, 5, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(goto_start);

	/* back button */	
	back = gtk_button_new();
	button_box_back = xpm_box(h264_window, (gchar **)xpm_back);
	gtk_container_add(GTK_CONTAINER(back), button_box_back);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "back", back,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(back), "clicked",
			GTK_SIGNAL_FUNC(h264_back), NULL);
	gtk_table_attach(GTK_TABLE(table), back, 5, 6, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(back);

	/* forward button */	
	forward = gtk_button_new();
	button_box_forward = xpm_box(h264_window, (gchar **)xpm_forward);
	gtk_container_add(GTK_CONTAINER(forward), button_box_forward);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "forward", forward,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(forward), "clicked",
			GTK_SIGNAL_FUNC(h264_forward), NULL);
	gtk_table_attach(GTK_TABLE(table), forward, 6, 7, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(forward);

	/* fast forward button */	
	goto_end = gtk_button_new();
	button_box_goto_end = xpm_box(h264_window, (gchar **)xpm_goto_end);
	gtk_container_add(GTK_CONTAINER(goto_end), button_box_goto_end);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "goto_end", goto_end,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(goto_end), "clicked",
			GTK_SIGNAL_FUNC(h264_goto_end), NULL);
	gtk_table_attach(GTK_TABLE(table), goto_end, 7, 8, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(goto_end);
	
	/* separate */	
	space = xpm_box(h264_window, (gchar **)xpm_space);
	gtk_table_attach(GTK_TABLE(table), space, 8, 9, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(space);

	/* step button */
	back = gtk_button_new_with_label("Back");
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "stepback", back,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(back), "clicked",
			GTK_SIGNAL_FUNC(h264_stepback), NULL);
	gtk_table_attach(GTK_TABLE(table), back, 9, 10, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(back);

	/* picture button */
	/* step button */
	step = gtk_button_new_with_label("Step");
	//gtk_object_set_data_full(GTK_OBJECT(h264_window), "step", step,
	//		(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(step), "clicked",
			GTK_SIGNAL_FUNC(h264_step), NULL);
	gtk_table_attach(GTK_TABLE(table), step, 10, 11, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(step);

	/* picture button */
	picture = gtk_button_new_with_label("PIC");
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "picture", picture,
			(GtkDestroyNotify)0);
	gtk_signal_connect(GTK_OBJECT(picture), "clicked",
			GTK_SIGNAL_FUNC(h264_picture), NULL);
	gtk_table_attach(GTK_TABLE(table), picture, 11, 12, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(picture);
	/* end of buttons operate */
	
	/* separate */	
	space = xpm_box(h264_window, (gchar **)xpm_space);
	gtk_table_attach(GTK_TABLE(table), space, 12, 13, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(space);

	audio = gtk_check_button_new_with_label("Audio");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(audio), TRUE);
	gtk_signal_connect(GTK_OBJECT(audio),"clicked",
			GTK_SIGNAL_FUNC(play_audio_callback), NULL);
	gtk_table_attach(GTK_TABLE(table), audio, 13, 15, 0, 1,
			(GtkAttachOptions)(0/*GTK_FILL*/),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(audio);
	
	
	label = gtk_label_new("Volume");
	gtk_table_attach(GTK_TABLE(table), label, 16, 17, 0, 1,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(label);

	
	volume = gtk_adjustment_new(100, 0, 100, 1, 0, 0);
	gtk_signal_connect(volume, "value_changed", GTK_SIGNAL_FUNC(ch_volume), NULL);
	volume_scale = gtk_hscale_new(GTK_ADJUSTMENT(volume));
	gtk_widget_set_usize(volume_scale, 80, 30);
	gtk_range_set_update_policy(GTK_RANGE(volume_scale), GTK_UPDATE_CONTINUOUS);
	gtk_scale_set_draw_value(GTK_SCALE(volume_scale), FALSE);
	//gtk_adjustment_set_value(GTK_ADJUSTMENT(volume), 50);

	gtk_table_attach(GTK_TABLE(table), volume_scale, 18, 21, 0, 1,
			(GtkAttachOptions)(GTK_FILL),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(volume_scale);
	
	
	/* separate */	
	space = xpm_box(h264_window, (gchar **)xpm_space);
	gtk_table_attach(GTK_TABLE(table), space, 0, 11, 1, 2,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(space);

	w_currenttime = gtk_text_new(NULL, NULL);
	gtk_widget_set_usize(w_currenttime, 70,20);
	gtk_text_set_editable(GTK_TEXT(w_currenttime), FALSE);
	//gtk_widget_ref(w_totaltime);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "currenttime", w_currenttime,
			(GtkDestroyNotify)0/*gtk_widget_unref*/);
	gtk_table_attach(GTK_TABLE(table), w_currenttime, 0, 3, 2, 3,
			(GtkAttachOptions)(0/*GTK_FILL*/),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(w_currenttime);
	gtk_text_insert(GTK_TEXT(w_currenttime), fixed_font/*NULL*/, NULL, NULL, "00:00:00", -1);
	
	seek = gtk_adjustment_new(0, 0, 100, 1, 0, 0);
	scale = gtk_hscale_new(GTK_ADJUSTMENT(seek));
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "scale", scale,
			(GtkDestroyNotify)0);
	gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
	gtk_table_attach(GTK_TABLE(table), scale, 3, 18, 2, 3,
			(GtkAttachOptions)(GTK_FILL),
			(GtkAttachOptions)(0),0,0);
	gtk_signal_connect(seek, "value_changed", GTK_SIGNAL_FUNC(ch_seek), NULL);
	gtk_widget_show(scale);

	w_totaltime = gtk_text_new(NULL, NULL);
	gtk_widget_set_usize(w_totaltime, 70,20);
	gtk_text_set_editable(GTK_TEXT(w_totaltime), FALSE);
	//gtk_widget_ref(w_totaltime);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "totaltime", w_totaltime,
			(GtkDestroyNotify)0/*gtk_widget_unref*/);
	gtk_table_attach(GTK_TABLE(table), w_totaltime, 19, 21, 2, 3,
			(GtkAttachOptions)(0/*GTK_FILL*/),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(w_totaltime);
	gtk_text_insert(GTK_TEXT(w_totaltime), fixed_font/*NULL*/, NULL, NULL, "00:00:00", -1);


	/* display frames */
	/* separate */
	space = xpm_box(h264_window, (gchar **)xpm_space);
	gtk_table_attach(GTK_TABLE(table), space, 0, 11, 3, 4,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(space);

	label = gtk_label_new("Current Frame");
	gtk_table_attach(GTK_TABLE(table), label, 0, 5, 4, 5,
			(GtkAttachOptions)(0/*GTK_FILL*/),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(label);

	w_frameplayed = gtk_text_new(NULL, NULL);
	gtk_widget_set_usize(w_frameplayed, 70,20);
	gtk_text_set_editable(GTK_TEXT(w_frameplayed), FALSE);
	//gtk_widget_ref(w_totaltime);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "frameplayed", w_frameplayed,
			(GtkDestroyNotify)0/*gtk_widget_unref*/);
	gtk_table_attach(GTK_TABLE(table), w_frameplayed, 5, 8, 4, 5,
			(GtkAttachOptions)(0/*GTK_FILL*/),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(w_frameplayed);
	tmp = 0;
	sprintf(buf, "%8d", tmp);
	gtk_text_insert(GTK_TEXT(w_frameplayed), fixed_font, NULL, NULL, buf, -1);
	
#if 1
	label = gtk_label_new("Total Frames");
	gtk_table_attach(GTK_TABLE(table), label, 8, 12, 4, 5,
			(GtkAttachOptions)(0),
			(GtkAttachOptions)(0),5,0);
	gtk_widget_show(label);
	
	w_totalframes = gtk_text_new(NULL, NULL);
	gtk_widget_set_usize(w_totalframes, 70,20);
	gtk_text_set_editable(GTK_TEXT(w_totalframes), FALSE);
	gtk_object_set_data_full(GTK_OBJECT(h264_window), "totalframes", w_totalframes,
			(GtkDestroyNotify)0);
	gtk_table_attach(GTK_TABLE(table), w_totalframes, 12, 15, 4, 5,
			(GtkAttachOptions)(0/*GTK_FILL*/),
			(GtkAttachOptions)(0),0,0);
	gtk_widget_show(w_totalframes);
	tmp = 0;
	sprintf(buf, "%8d", tmp);
	gtk_text_insert(GTK_TEXT(w_totalframes), fixed_font, NULL, NULL, buf, -1);
#endif
	return h264_window ;
}

int main(int argc, char *argv[])
{
	gtk_init(&argc, &argv);

	h264_create_window();

	h264_set_buttons_sensitive(0);

	if(!using_sdl){
		videofd = creat("video.yuv", 0777);
	}
		
	gtk_main();

	return 0;
}
